@@ -28,6 +28,13 @@
 
 #define TRACKER_MONITOR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_MONITOR, TrackerMonitorPrivate))
 
+/* If blacklisting enabled, CREATED events won't be directly notified, and
+ * instead, they will be pushed to the events cache; so that if a DELETED event
+ * comes during the CACHE_LIFETIME of the event, both will cancel each other and
+ * no event will be notified. This also enables some additional event mergings
+ * that can be done with MOVED events. */
+#define ENABLE_FILE_BLACKLISTING TRUE
+
 /* The life time of an item in the cache */
 #define CACHE_LIFETIME_SECONDS 1
 
@@ -81,6 +88,7 @@ typedef struct {
 enum {
 	ITEM_CREATED,
 	ITEM_UPDATED,
+	ITEM_ATTRIBUTE_UPDATED,
 	ITEM_DELETED,
 	ITEM_MOVED,
 	LAST_SIGNAL
@@ -147,6 +155,17 @@ tracker_monitor_class_init (TrackerMonitorClass *klass)
 		              2,
 		              G_TYPE_OBJECT,
 		              G_TYPE_BOOLEAN);
+	signals[ITEM_ATTRIBUTE_UPDATED] =
+		g_signal_new ("item-attribute-updated",
+		              G_TYPE_FROM_CLASS (klass),
+		              G_SIGNAL_RUN_LAST,
+		              0,
+		              NULL, NULL,
+		              tracker_marshal_VOID__OBJECT_BOOLEAN,
+		              G_TYPE_NONE,
+		              2,
+		              G_TYPE_OBJECT,
+		              G_TYPE_BOOLEAN);
 	signals[ITEM_DELETED] =
 		g_signal_new ("item-deleted",
 		              G_TYPE_FROM_CLASS (klass),
@@ -613,7 +632,6 @@ emit_signal_for_event (TrackerMonitor *monitor,
 
 	case G_FILE_MONITOR_EVENT_CHANGED:
 	case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
-	case G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED:
 		g_debug ("Emitting ITEM_UPDATED for (%s) '%s'",
 		         event_data->is_directory ? "DIRECTORY" : "FILE",
 		         event_data->file_uri);
@@ -623,6 +641,16 @@ emit_signal_for_event (TrackerMonitor *monitor,
 			       event_data->is_directory);
 		break;
 
+	case G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED:
+		g_debug ("Emitting ITEM_ATTRIBUTE_UPDATED for (%s) '%s'",
+		         event_data->is_directory ? "DIRECTORY" : "FILE",
+		         event_data->file_uri);
+		g_signal_emit (monitor,
+			       signals[ITEM_ATTRIBUTE_UPDATED], 0,
+			       event_data->file,
+			       event_data->is_directory);
+		break;
+
 	case G_FILE_MONITOR_EVENT_DELETED:
 		g_debug ("Emitting ITEM_DELETED for (%s) '%s'",
 		         event_data->is_directory ? "DIRECTORY" : "FILE",
@@ -740,10 +768,10 @@ event_pairs_timeout_cb (gpointer user_data)
 
 static void
 monitor_event_cb (GFileMonitor	    *file_monitor,
-		  GFile		    *file,
-		  GFile		    *other_file,
-		  GFileMonitorEvent  event_type,
-		  gpointer	     user_data)
+                  GFile		    *file,
+                  GFile		    *other_file,
+                  GFileMonitorEvent  event_type,
+                  gpointer	     user_data)
 {
 	TrackerMonitor *monitor;
 	gchar *file_uri;
@@ -821,23 +849,37 @@ monitor_event_cb (GFileMonitor	    *file_monitor,
 				/* Get previous event data, if any */
 				previous_update_event_data = g_hash_table_lookup (monitor->private->pre_update, file);
 
-				/* If there is a previous ATTRIBUTE_CHANGED still not notified,
-				 * remove it, as we know there will be a CHANGES_DONE_HINT afterwards
-				 */
-				if (previous_update_event_data &&
-				    previous_update_event_data->event_type == G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED) {
-					g_hash_table_remove (monitor->private->pre_update, file);
+				if (previous_update_event_data) {
+					if (previous_update_event_data->event_type == G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED) {
+						/* If there is a previous ATTRIBUTE_CHANGED still not notified,
+						 * remove it, as we know there will be a CHANGES_DONE_HINT afterwards
+						 */
+						g_hash_table_remove (monitor->private->pre_update, file);
+					} else if (previous_update_event_data->event_type == G_FILE_MONITOR_EVENT_CREATED) {
+						/* Update the start_time of the original CREATED event that we're refreshing until
+						 * there is a CHANGES_DONE_HINT. */
+						g_get_current_time (&(previous_update_event_data->start_time));
+					}
+					break;
 				}
-
-				break;
 			}
 		} /* Else, Fall through and treat as an ATTRIBUTE_CHANGED */
 
 		case G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED: {
-			if (!g_hash_table_lookup (monitor->private->pre_update, file)) {
+			EventData *previous_update_event_data;
+
+			/* Get previous event data, if any */
+			previous_update_event_data = g_hash_table_lookup (monitor->private->pre_update, file);
+			if (!previous_update_event_data) {
+				/* If no previous one, insert it */
 				g_hash_table_insert (monitor->private->pre_update,
 				                     g_object_ref (file),
 				                     event_data_new (file, NULL, FALSE, event_type));
+			} else {
+				/* Update the start_time of the previous one.
+				 * This could be the original CREATED event that we're refreshing until
+				 * there is a CHANGES_DONE_HINT. */
+				g_get_current_time (&(previous_update_event_data->start_time));
 			}
 
 			break;
@@ -849,6 +891,18 @@ monitor_event_cb (GFileMonitor	    *file_monitor,
 			/* Get previous event data, if any */
 			previous_update_event_data = g_hash_table_lookup (monitor->private->pre_update, file);
 
+#if ENABLE_FILE_BLACKLISTING
+			if (previous_update_event_data) {
+				/* Refresh event timer */
+				g_get_current_time (&(previous_update_event_data->start_time));
+			} else {
+				/* Insert new update item in cache */
+				g_hash_table_insert (monitor->private->pre_update,
+				                     g_object_ref (file),
+				                     event_data_new (file, NULL, FALSE, event_type));
+			}
+#else
+			/* If no blacklisting desired, emit event right away */
 			if (previous_update_event_data) {
 				emit_signal_for_event (monitor, previous_update_event_data);
 				g_hash_table_remove (monitor->private->pre_update, file);
@@ -859,28 +913,106 @@ monitor_event_cb (GFileMonitor	    *file_monitor,
 				emit_signal_for_event (monitor, new_event);
 				event_data_free (new_event);
 			}
-
+#endif /* ENABLE_FILE_BLACKLISTING */
 			break;
 		}
 
 		case G_FILE_MONITOR_EVENT_DELETED: {
 			EventData *new_event;
+#if ENABLE_FILE_BLACKLISTING
+			EventData *previous_update_event_data;
+
+			/* Get previous event data, if any */
+			previous_update_event_data = g_hash_table_lookup (monitor->private->pre_update, file);
+
+			/* Remove any previous pending event (if blacklisting enabled, there may be some) */
+			if (previous_update_event_data) {
+				GFileMonitorEvent previous_update_event_type;
+
+				previous_update_event_type = previous_update_event_data->event_type;
+				g_hash_table_remove (monitor->private->pre_update, file);
+
+				if (previous_update_event_type == G_FILE_MONITOR_EVENT_CREATED) {
+					/* Oh, oh, oh, we got a previous CREATED event waiting in the event
+					 * cache... so we cancel it with the DELETED and don't notify anything */
+					break;
+				}
+				/* else, keep on notifying the event */
+			}
+#endif /* ENABLE_FILE_BLACKLISTING */
 
 			new_event = event_data_new (file, NULL, FALSE, event_type);
 			emit_signal_for_event (monitor, new_event);
 			event_data_free (new_event);
-
 			break;
 		}
+
 		case G_FILE_MONITOR_EVENT_MOVED: {
 			EventData *new_event;
+#if ENABLE_FILE_BLACKLISTING
+			EventData *previous_update_event_data;
+
+			/* Get previous event data, if any */
+			previous_update_event_data = g_hash_table_lookup (monitor->private->pre_update, file);
+
+			/* Some event-merging that can also be enabled if doing blacklisting, as we are
+			 * queueing the CHANGES_DONE_HINT in the cache :
+			 *   (a) CREATED(A)      + MOVED(A->B)  = CREATED (B)
+			 *   (b) UPDATED(A)      + MOVED(A->B)  = MOVED(A->B) + UPDATED(B)
+			 *   (c) ATTR_UPDATED(A) + MOVED(A->B)  = MOVED(A->B) + UPDATED(B)
+			 * Notes:
+			 *  - In case (a), the CREATED event is queued in the cache.
+			 *  - In case (a), note that B may already exist before, so instead of a CREATED
+			 *    we should be issuing an UPDATED instead... we don't do it as at the end we
+			 *    don't really mind, and we save a call to g_file_query_exists().
+			 *  - In case (b), the MOVED event is emitted right away, while the UPDATED
+			 *    one is queued in the cache.
+			 *  - In case (c), we issue an UPDATED instead of ATTR_UPDATED, because there may
+			 *    already be another UPDATED or CREATED event for B, so we make sure in this
+			 *    way that not only attributes get checked at the end.
+			 * */
+			if (previous_update_event_data) {
+				if (previous_update_event_data->event_type == G_FILE_MONITOR_EVENT_CREATED) {
+					/* (a) CREATED(A) + MOVED(A->B)  = CREATED (B)
+					 *
+					 * Oh, oh, oh, we got a previous created event
+					 * waiting in the event cache... so we remove it, and we
+					 * convert the MOVE event into a CREATED(other_file) */
+					g_hash_table_remove (monitor->private->pre_update, file);
+					g_hash_table_replace (monitor->private->pre_update,
+					                      g_object_ref (other_file),
+					                      event_data_new (other_file,
+					                                      NULL,
+					                                      FALSE,
+					                                      G_FILE_MONITOR_EVENT_CREATED));
+
+					/* Do not notify the moved event now */
+					break;
+				} else {
+					/*   (b) UPDATED(A)      + MOVED(A->B)  = MOVED(A->B) + UPDATED(B)
+					 *   (c) ATTR_UPDATED(A) + MOVED(A->B)  = MOVED(A->B) + UPDATED(B)
+					 *
+					 * We setup here the UPDATED(B) event, added to the cache */
+					g_hash_table_replace (monitor->private->pre_update,
+					                      g_object_ref (other_file),
+					                      event_data_new (other_file,
+					                                      NULL,
+					                                      FALSE,
+					                                      G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT));
+					/* Remove previous event */
+					g_hash_table_remove (monitor->private->pre_update, file);
+
+					/* And keep on notifying the MOVED event */
+				}
+			}
+#endif /* ENABLE_FILE_BLACKLISTING */
 
 			new_event = event_data_new (file, other_file, FALSE, event_type);
 			emit_signal_for_event (monitor, new_event);
 			event_data_free (new_event);
-
 			break;
 		}
+
 		case G_FILE_MONITOR_EVENT_PRE_UNMOUNT:
 		case G_FILE_MONITOR_EVENT_UNMOUNTED:
 			/* Do nothing */
