@@ -57,15 +57,19 @@ typedef struct {
 	GArray *generic_extractors;
 	gboolean disable_shutdown;
 	gboolean force_internal_extractors;
+	gboolean disable_summary_on_finalize;
 } TrackerExtractPrivate;
 
 typedef struct {
 	const GModule *module;
 	const TrackerExtractData *edata;
-}  ModuleData;
-
+	GPatternSpec *pattern; /* For a fast g_pattern_match() */
+	guint extracted_count;
+	guint failed_count;
+} ModuleData;
 
 static void tracker_extract_finalize (GObject *object);
+static void report_statistics        (GObject *object);
 
 G_DEFINE_TYPE(TrackerExtract, tracker_extract, G_TYPE_OBJECT)
 
@@ -93,19 +97,111 @@ static void
 tracker_extract_finalize (GObject *object)
 {
 	TrackerExtractPrivate *priv;
+	gint i;
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
 
+	if (!priv->disable_summary_on_finalize) {
+		report_statistics (object);
+	}
+
 #ifdef HAVE_LIBSTREAMANALYZER
 	tracker_topanalyzer_shutdown ();
 #endif /* HAVE_STREAMANALYZER */
 
+	for (i = 0; i < priv->specific_extractors->len; i++) {
+		ModuleData *mdata;
+
+		mdata = &g_array_index (priv->specific_extractors, ModuleData, i);
+		g_pattern_spec_free (mdata->pattern);
+	}
 	g_array_free (priv->specific_extractors, TRUE);
+
+	for (i = 0; i < priv->generic_extractors->len; i++) {
+		ModuleData *mdata;
+
+		mdata = &g_array_index (priv->generic_extractors, ModuleData, i);
+		g_pattern_spec_free (mdata->pattern);
+	}
 	g_array_free (priv->generic_extractors, TRUE);
 
 	G_OBJECT_CLASS (tracker_extract_parent_class)->finalize (object);
 }
 
+static void
+report_statistics (GObject *object)
+{
+	TrackerExtractPrivate *priv;
+	GHashTable *reported = NULL;
+	gint i;
+
+	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
+
+	g_message ("--------------------------------------------------");
+	g_message ("Statistics:");
+	g_message ("  Specific Extractors:");
+
+	reported = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+	for (i = 0; i < priv->specific_extractors->len; i++) {
+		ModuleData *mdata;
+		const gchar *name;
+
+		mdata = &g_array_index (priv->specific_extractors, ModuleData, i);
+		name = g_module_name ((GModule*) mdata->module);
+
+		if ((mdata->extracted_count > 0 || mdata->failed_count > 0) &&
+		    !g_hash_table_lookup (reported, name)) {
+			const gchar *name_without_path;
+
+			name_without_path = strrchr (name, G_DIR_SEPARATOR) + 1;
+
+			g_message ("    Module:'%s', extracted:%d, failures:%d",
+			           name_without_path,
+			           mdata->extracted_count,
+			           mdata->failed_count);
+			g_hash_table_insert (reported, (gpointer) name, GINT_TO_POINTER(1));
+		}
+	}
+
+	if (g_hash_table_size (reported) < 1) {
+		g_message ("    No files handled");
+	}
+
+	g_hash_table_remove_all (reported);
+
+	g_message ("  Generic Extractors:");
+
+	for (i = 0; i < priv->generic_extractors->len; i++) {
+		ModuleData *mdata;
+		const gchar *name;
+
+		mdata = &g_array_index (priv->generic_extractors, ModuleData, i);
+		name = g_module_name ((GModule*) mdata->module);
+
+		if ((mdata->extracted_count > 0 || mdata->failed_count > 0) &&
+		    !g_hash_table_lookup (reported, name)) {
+			const gchar *name_without_path;
+
+			name_without_path = strrchr (name, G_DIR_SEPARATOR) + 1;
+
+			g_message ("    Module:'%s', extracted:%d, failed:%d",
+			           name_without_path,
+			           mdata->extracted_count,
+			           mdata->failed_count);
+			g_hash_table_insert (reported, (gpointer) name, GINT_TO_POINTER(1));
+		}
+	}
+
+	if (g_hash_table_size (reported) < 1) {
+		g_message ("    No files handled");
+	}
+
+	g_message ("--------------------------------------------------");
+
+	g_hash_table_unref (reported);
+}
+
 static gboolean
 load_modules (const gchar  *force_module,
               GArray      **specific_extractors,
@@ -201,6 +297,9 @@ load_modules (const gchar  *force_module,
 			           g_module_name ((GModule*) mdata.module));
 
 			for (; mdata.edata->mime; mdata.edata++) {
+				/* Compile pattern from mime */
+				mdata.pattern = g_pattern_spec_new (mdata.edata->mime);
+
 				if (G_UNLIKELY (strchr (mdata.edata->mime, '*') != NULL)) {
 					g_message ("  Generic  match for mime:'%s'",
 					           mdata.edata->mime);
@@ -380,21 +479,31 @@ get_file_metadata (TrackerExtract         *extract,
 	 */
 	if (mime_used) {
 		guint i;
+		glong length;
+		gchar *reversed;
+
+		/* Using a reversed string while pattern matching is faster
+		 * if we have lots of patterns with wildcards.
+		 * We are assuming here that mime_used is ASCII always, so
+		 * we avoid g_utf8_strreverse() */
+		reversed = g_strdup (mime_used);
+		g_strreverse (reversed);
+		length = strlen (mime_used);
 
 		for (i = 0; i < priv->specific_extractors->len; i++) {
 			const TrackerExtractData *edata;
-			ModuleData mdata;
+			ModuleData *mdata;
 
-			mdata = g_array_index (priv->specific_extractors, ModuleData, i);
-			edata = mdata.edata;
+			mdata = &g_array_index (priv->specific_extractors, ModuleData, i);
+			edata = mdata->edata;
 
-			if (g_pattern_match_simple (edata->mime, mime_used)) {
+			if (g_pattern_match (mdata->pattern, length, mime_used, reversed)) {
 				gint items;
 
 				tracker_dbus_request_comment (request_id,
 				                              context,
 				                              "  Extracting with module:'%s'",
-				                              g_module_name ((GModule*) mdata.module));
+				                              g_module_name ((GModule*) mdata->module));
 
 				(*edata->func) (uri, preupdate, statements);
 
@@ -404,35 +513,39 @@ get_file_metadata (TrackerExtract         *extract,
 				                              context,
 				                              "  Found %d metadata items",
 				                              items);
+
+				mdata->extracted_count++;
+
 				if (items == 0) {
+					mdata->failed_count++;
 					continue;
 				}
 
 				tracker_sparql_builder_insert_close (statements);
 
 				g_free (mime_used);
+				g_free (reversed);
 
 				*preupdate_out = preupdate;
 				*statements_out = statements;
-
 				return TRUE;
 			}
 		}
 
 		for (i = 0; i < priv->generic_extractors->len; i++) {
 			const TrackerExtractData *edata;
-			ModuleData mdata;
+			ModuleData *mdata;
 
-			mdata = g_array_index (priv->generic_extractors, ModuleData, i);
-			edata = mdata.edata;
+			mdata = &g_array_index (priv->generic_extractors, ModuleData, i);
+			edata = mdata->edata;
 
-			if (g_pattern_match_simple (edata->mime, mime_used)) {
+			if (g_pattern_match (mdata->pattern, length, mime_used, reversed)) {
 				gint items;
 
 				tracker_dbus_request_comment (request_id,
 				                              context,
 				                              "  Extracting with module:'%s'",
-				                              g_module_name ((GModule*) mdata.module));
+				                              g_module_name ((GModule*) mdata->module));
 
 				(*edata->func) (uri, preupdate, statements);
 
@@ -442,13 +555,18 @@ get_file_metadata (TrackerExtract         *extract,
 				                              context,
 				                              "  Found %d metadata items",
 				                              items);
+
+				mdata->extracted_count++;
+
 				if (items == 0) {
+					mdata->failed_count++;
 					continue;
 				}
 
 				tracker_sparql_builder_insert_close (statements);
 
 				g_free (mime_used);
+				g_free (reversed);
 
 				*preupdate_out = preupdate;
 				*statements_out = statements;
@@ -457,11 +575,14 @@ get_file_metadata (TrackerExtract         *extract,
 			}
 		}
 
-		g_free (mime_used);
-
 		tracker_dbus_request_comment (request_id,
 		                              context,
-		                              "  Could not find any extractors to handle metadata type");
+		                              "  Could not find any extractors to handle metadata type "
+		                              "(mime: %s)",
+		                              mime_used);
+
+		g_free (mime_used);
+		g_free (reversed);
 	} else {
 		tracker_dbus_request_comment (request_id,
 		                              context,
@@ -485,9 +606,13 @@ tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
 {
 	guint request_id;
 	TrackerSparqlBuilder *statements, *preupdate;
+	TrackerExtractPrivate *priv;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
+	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
+	priv->disable_summary_on_finalize = TRUE;
+
 	g_return_if_fail (uri != NULL);
 
 	tracker_dbus_request_new (request_id,
@@ -625,7 +750,6 @@ tracker_extract_get_metadata (TrackerExtract         *object,
 	}
 }
 
-#ifdef HAVE_DBUS_FD_PASSING
 static void
 get_metadata_fast (TrackerExtract *object,
                    DBusConnection *connection,
@@ -841,4 +965,3 @@ tracker_extract_connection_filter (DBusConnection *connection,
 
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
-#endif /* HAVE_DBUS_FD_PASSING */
