@@ -61,11 +61,9 @@ void            tracker_extract_get_metadata            (TrackerExtract
                                                          const gchar            *mime,
                                                          DBusGMethodInvocation  *context,
                                                          GError                **error);
-#ifdef HAVE_DBUS_FD_PASSING
 DBusHandlerResult tracker_extract_connection_filter     (DBusConnection         *connection,
                                                          DBusMessage            *message,
                                                          void                   *user_data);
-#endif /* HAVE_DBUS_FD_PASSING */
 
 /* Not DBus API */
 void            tracker_extract_get_metadata_by_cmdline (TrackerExtract         *object,
